/* Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"
// global defines : OUTPUT_MUX_TRI

//`define OUTPUT_MUX_TRI

// Module Name: rtavr_ior_port
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                          DO:  NORMAL    TRISTATE(FAKE)   
// Number of Slice Flip Flops:     16         16
// Number of 4 input LUTs:         22         23
// Number of occupied Slices:      19         20
// Total Number of 4 input LUTs:   22         23 
// Number of bonded IOBs :         33         33
//   IOB Flip Flops :              16         16
// Number of BUFGMUXs :             1          1
// Number of RAMB16BWEs :          -           -

module rtavr_ior_port(
		input CLK,
		input RESET,

		input [7:0] DI,
		output [7:0] DO
		, input WR,RD
		, input [5:0] ADDR
		// external I/O
		, inout [7:0] PORT
		);
  parameter BASE_ADDR = 0; // PORTA 0x00
//  parameter BASE_ADDR = 4; // PORTB 0x04
//  parameter BASE_ADDR = 27; // PORTC 0x1b

  reg  [7:0] r_pin;
  reg  [7:0] r_ddr;
  reg  [7:0] r_port;

  wire OE_pin = RD & (ADDR == BASE_ADDR);
  //wire WE_pin = WR & (ADDR == BASE_ADDR);
  wire OE_ddr = RD & (ADDR == (BASE_ADDR + 1)); 
  wire WE_ddr = WR & (ADDR == (BASE_ADDR + 1));
  wire OE_port = RD & (ADDR == (BASE_ADDR + 2)); 
  wire WE_port = WR & (ADDR == (BASE_ADDR + 2));

`ifdef OUTPUT_MUX_TRI
  wire [7:0] i_DO = OE_ddr ? r_ddr : OE_pin ? r_pin : OE_port ? r_port : 0;
  bufif1(DO, i_DO, (OE_pin | OE_ddr | OE_port));
`else
  wire [7:0] i_DO = OE_ddr ? r_ddr : OE_pin ? r_pin : OE_port ? r_port : 0;
  assign DO = i_DO;
`endif
  always @(negedge CLK)
  begin
    if (RESET == 0)
      begin
	r_pin <= 0;
      end
    else
      begin
	r_pin <= PORT;
      end
  end
  always @(posedge CLK)
  begin
    if (RESET == 0)
      begin
	r_ddr <= 0;
	r_port <= 0;
      end
    else
      begin
	//if(WE_pin) r_pin <= DI;
	if(WE_ddr) r_ddr <= DI;
	if(WE_port) r_port <= DI;
      end
  end

  generate 
    genvar i;
    for (i=0; i<8; i=i+1)
       begin : port_out
	   bufif1(PORT[i],r_port[i],r_ddr[i]);
       end
  endgenerate
endmodule
