`timescale 10ps / 10ps
`define GTK // use VCD

`include "rtavr_defs.v"

module tb_001_nop;

   reg CLK2X;  // input
   reg RESET;  // input
   wire [11:0] ROM_ADDRA; // output
   reg [15:0] ROM_DOA; // input
   wire ROM_WEB; // output
   wire ROM_OE; // output
   wire [12:0] ROM_ADDRB; // output
   wire [7:0] ROM_DIB; // output
   wire [7:0] ROM_DOB = 0; // input
   wire [4:0] INT_VEC; // output
   wire INT_REQ; // output
   wire INT_ACK = 0; // input

`ifdef SIM_OUTPUT_S1
   wire [2:0] SIM_OP3_SEL;  // output
   wire [2:0] SIM_OP2_SEL;  // output
   wire [2:0] SIM_OP1_SEL;  // output
   wire [7:0] SIM_IMM_DATA;  // output
   wire [2:0] SIM_SBIX_BIT;  // output
   wire [1:0] SIM_PUSH_SEL;  // output
`endif
rtavr top (.RESET(RESET), .CLK2X(CLK2X)
		, .ROM_ADDRA(ROM_ADDRA[11:0]) 
		, .ROM_DOA(ROM_DOA[15:0])
		, .ROM_WEB(ROM_WEB), .ROM_OE(ROM_OE)
		, .ROM_ADDRB(ROM_ADDRB[12:0]) 
		, .ROM_DIB(ROM_DIB[7:0])
		, .ROM_DOB(ROM_DOB[7:0])
		, .INT_VEC(INT_VEC[4:0])
		, .INT_REQ(INT_REQ)
		, .INT_ACK(INT_ACK)
`ifdef SIM_OUTPUT_S0
		, .SIM_WB_INST(S0_WB_INST)
		, .SIM_PREDEC(GPR_PREDEC)
		, .SIM_POSTINC(GPR_POSTINC)
`endif
`ifdef SIM_OUTPUT_S1
		, .SIM_LOAD(CMD_LOAD)
		, .SIM_LOAD_WR(CMD_LOAD_WR)
		, .SIM_STORE(CMD_STORE)
		, .SIM_OP3(CMD_OP3)
		, .SIM_OP3_WR(CMD_OP3_WR)
		, .SIM_OP2(CMD_OP2)
		, .SIM_OP1(CMD_OP1)
		, .SIM_SBIX(CMD_SBIX)
		, .SIM_SBRX(CMD_SBRX)
		, .SIM_OP3_WC(OP3_WC)
		, .SIM_OP3_SEL(OP3_SEL[2:0])
		, .SIM_OP2_SEL(OP2_SEL[2:0])
		, .SIM_OP1_SEL(OP1_SEL[2:0])
		, .SIM_IMM_DATA(IMM_DATA[7:0])
		, .SIM_SBIX_BIT(SBIX_BIT[2:0])
		, .SIM_SBIX_BIT_OUT(SBRX_BIT_OUT)
		, .SIM_EA_ABS(EA_ABS)
		, .SIM_EA_PUSH(EA_PUSH)
		, .SIM_EA_POP(EA_POP)
		, .SIM_PUSH_SEL(PUSH_SEL[1:0])
`endif
 );

   parameter CLKP = 5000;        //  50ns
   parameter CLKH = CLKP/2;      //
   parameter CLKL = CLKP-CLKH;   //
   
   always begin
      #CLKH CLK2X = 0;
      #CLKL CLK2X = 1;
   end

`ifdef GTK
   initial begin
      $dumpfile("tb_001_nop.vcd");
      $dumpvars(0,tb_001_nop.top);
   end
`endif

   initial begin : test
      sys_reset;
      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP
      $display("*** Simulation Complete! ***");
      $finish;
   end

   task sys_reset;
      begin
         #1 RESET = 0;
         CLK2X = 0;
         repeat(3) @(posedge CLK2X);
         #1 RESET = 1;
         @(posedge CLK2X);
      end
   endtask // sys_reset
   
   task rom_out;
      input [15:0] data;
      begin
	 @(posedge CLK2X);
	 @(negedge CLK2X);
         #1 ROM_DOA[15:0] = data[15:0];
         @(posedge CLK2X);
      end
   endtask // rom_out

endmodule // test
