// GPR modele for the Reduced Core TinyAVR (attiny10) 
// version 005 -- remove test code
// 
// Target Device: xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal: Balanced
// LATCH_DOL : Insert LATCHes for Timing
//                                  NORMAL  FORWARD_DI
// Number of Slice Flip Flops:        61         61
// Number of 4 input LUTs:            93        118
// Number of occupied Slices:         70         73
// Total Number of 4 input LUTs:     100        125
// Number used for Dual Port RAMs:    16         16
// Number of bonded IOBs:             62         62
// Post-PAR Static Timing Report
// Clock to Setup                  8.657      9.440

`define USE_DMY_CLOCK
//`define OUTPUT_WB_ADDR
`define FOWARD_DI
`define CHECK_CONFLICT

module rtavr_gpr_16(
		input CLK2X,
`ifdef USE_DMY_CLOCK
		output CLK_OUT,
`else
		input CLK,
`endif
		input WE,
		input PREDEC, POSTINC, 
		input [7:0] DI,
		input [3:0] ADDRAL,
		input [3:0] ADDRAH,
		input [3:0] ADDRBL,
		input [3:0] ADDRBH,
		output [7:0] DOAL,
		output [7:0] DOAH,
		output [7:0] DOBL,
		output [7:0] DOBH
`ifdef CHECK_CONFLICT
		, output DI_CONFLICT
`endif
		, output WB_VALID

`ifdef OUTPUT_WB_ADDR
		, output [3:0] WB_ADDR
`endif
		);
// Timing MEMO:
//
// CLK2X    1       0       1        0        1       0        1       0
// CLK           1               0                1                0
//      |  ALL input    |   ALL input     |               |                |
//      |               |   (no change)   |               |                |
//                                        |   WB_VALID , WB_ADDR           |
//                                        |  DOAH , DOBH                   |
//    -- TEST6          |   DOAL,DOBL,                    |
//       TEST7                            | DOAL,DOBL,                     |
`ifdef USE_DMY_CLOCK
  reg CLK;
  assign CLK_OUT = CLK;
`endif

  reg  [7:0] i_DOAL;
  reg  [7:0] i_DOAH;
  reg  [7:0] i_DOBL;
  reg  [7:0] i_DOBH;
  reg  [7:0] gpr [0:15];

  assign DOAL = i_DOAL;
  assign DOAH = i_DOAH;
  assign DOBL = i_DOBL;
  assign DOBH = i_DOBH;

  wire [3:0] ADDRA = (CLK == 1) ? ADDRAL : ADDRAH;
  wire [3:0] ADDRB = (CLK == 1) ? ADDRBL : ADDRBH;
  wire [8:0] DOA = { 0, gpr[ADDRA] };

  reg  c;
  reg  i_wb_valid;
`ifdef CHECK_CONFLICT
  assign DI_CONFLICT = WE & (ADDRAL == ADDRAH);
  wire WB = (PREDEC | POSTINC) & ~DI_CONFLICT;
`else
  wire WB = (PREDEC | POSTINC);
`endif
  `ifdef OUTPUT_WB_ADDR
    reg  [3:0] i_wb_addr;
    assign WB_ADDR = i_wb_addr;
  `endif
  reg  [7:0] i_wb_data;

  wire [8:0] DOA2 =  PREDEC ? DOA-(~CLK | c) : DOA;
  wire [8:0] WB_DATA = POSTINC ? DOA+(~CLK | c): DOA2;
  
  wire WE2 = (CLK == 0) ? (WE | i_wb_valid) : WB ;
  wire [7:0] DI2 = (WE == 1) ? DI : (i_wb_valid == 1) ? i_wb_data: WB_DATA ;
  assign WB_VALID = i_wb_valid;

  reg  [7:0] i0_DOAL;
  reg  [7:0] i0_DOBL;

`ifdef FOWARD_DI
  wire FORWARD_TO_BL = (WE2) & (ADDRAH == ADDRBL);
  wire FORWARD_TO_BH = (WE2) & (ADDRAH == ADDRBH);
`endif

`ifdef USE_DMY_CLOCK
always @(posedge CLK2X)
  begin
     CLK <= ~CLK;
  end
`endif

always @(posedge CLK2X)
  begin

     if (CLK == 1) i0_DOAL <= DOA2[7:0];
     if (CLK == 1) i0_DOBL <= gpr[ADDRB];

`ifdef FOWARD_DI
     if (CLK == 0) i_DOAL <= i0_DOAL;
     if (CLK == 0) i_DOAH <= DOA2[7:0];
     if (CLK == 0) i_DOBL <= FORWARD_TO_BL ? DI2 : i0_DOBL;
     if (CLK == 0) i_DOBH <= FORWARD_TO_BH ? DI2 : gpr[ADDRB];
`else
     if (CLK == 0) i_DOAL <= i0_DOAL;
     if (CLK == 0) i_DOAH <= DOA2[7:0];
     if (CLK == 0) i_DOBL <= i0_DOBL;
     if (CLK == 0) i_DOBH <= gpr[ADDRB];
`endif
     if (CLK == 0) i_wb_valid <= WB & c;	  
     if (CLK == 0) i_wb_data <= WB_DATA[7:0];
  `ifdef OUTPUT_WB_ADDR
     if (CLK == 0) i_wb_addr <= ADDRAH;
  `endif
     if (WE2) gpr[ADDRA] <= DI2;
     c <= WB_DATA[8];
  end
endmodule
